#include "s3e.h"
#include "IwDebug.h"
#include "Iw2DSceneGraph.h"
#include "IwGx.h"

#include "Application.h"

extern bool g_QuitGame;

extern CIwColour clearColour;
extern CIwColour blackColour;
extern CIwColour whiteColour;
extern CIwColour blueColour;
extern CIwColour redColour;
extern CIwColour yellowColour;
extern CIwColour greenColour;

Application::Application()
{
	titleImage = Iw2DCreateImage("titlescreen.png");
	//titleImage->GetMaterial()->SetClamping(false);
	//titleImage->GetMaterial()->SetFiltering(false);

	tileImage = Iw2DCreateImage("tiles.png");
	tileImage->GetMaterial()->SetClamping(false);
	tileImage->GetMaterial()->SetFiltering(false);
	tilesPerImageWidth = (uint32)(tileImage->GetWidth() / TILE_WIDTH);

	atariFont12 = Iw2DCreateFont("atariFont12.gxfont");
	atariFont16 = Iw2DCreateFont("atariFont16.gxfont");
	atariFont36 = Iw2DCreateFont("atariFont36.gxfont");

	currentLevel = NULL;
	player = NULL;

	startButtonSourceNormal.x = 0.0f;
	startButtonSourceNormal.y = 384.0f;
	startButtonSourceHovered.x = 96.0f;
	startButtonSourceHovered.y = 384.0f;
	startButtonTarget.x = 272.0f;
	startButtonTarget.y = 200.0f;
	startButtonSize.x = 96.0f;
	startButtonSize.y = 24.0f;
	isStartButtonHovered = false;

	quitButtonSourceNormal.x = 0.0f;
	quitButtonSourceNormal.y = 408.0f;
	quitButtonSourceHovered.x = 96.0f;
	quitButtonSourceHovered.y = 408.0f;
	quitButtonTarget.x = 272.0f;
	quitButtonTarget.y = 230.0f;
	quitButtonSize.x = 96.0f;
	quitButtonSize.y = 24.0f;
	isQuitButtonHovered = false;

	titlesButtonSourceNormal.x = 192.0f;
	titlesButtonSourceNormal.y = 384.0f;
	titlesButtonSourceHovered.x = 288.0f;
	titlesButtonSourceHovered.y = 384.0f;
	titlesButtonTarget.x = 272.0f;
	titlesButtonTarget.y = 200.0f;
	titlesButtonSize.x = 96.0f;
	titlesButtonSize.y = 24.0f;
	isTitlesButtonHovered = false;

	continueButtonSourceNormal.x = 192.0f;
	continueButtonSourceNormal.y = 408.0f;
	continueButtonSourceHovered.x = 288.0f;
	continueButtonSourceHovered.y = 408.0f;
	continueButtonTarget.x = 272.0f;
	continueButtonTarget.y = 230.0f;
	continueButtonSize.x = 96.0f;
	continueButtonSize.y = 24.0f;
	isContinueButtonHovered = false;
}

Application::~Application()
{
	if (player != NULL)
	{
		delete player;
	}

	if (currentLevel != NULL)
	{
		delete currentLevel;
	}

	delete atariFont36;
	delete atariFont16;
	delete atariFont12;

	delete tileImage;
	delete titleImage;
}

void Application::Init()
{
	QuitToTitles();
}

void Application::Update()
{
	int32  pointerX = s3ePointerGetX();
	int32  pointerY = s3ePointerGetY();

	bool lmb = s3ePointerGetState(S3E_POINTER_BUTTON_LEFTMOUSE) &  S3E_POINTER_STATE_DOWN;
	bool mmb = s3ePointerGetState(S3E_POINTER_BUTTON_MIDDLEMOUSE) &  S3E_POINTER_STATE_DOWN;
	bool rmb = s3ePointerGetState(S3E_POINTER_BUTTON_RIGHTMOUSE) &  S3E_POINTER_STATE_DOWN;

	bool lmbr = (s3ePointerGetState(S3E_POINTER_BUTTON_LEFTMOUSE) &  S3E_POINTER_STATE_RELEASED) == S3E_POINTER_STATE_RELEASED;
	bool mmbr = (s3ePointerGetState(S3E_POINTER_BUTTON_MIDDLEMOUSE) &  S3E_POINTER_STATE_RELEASED) == S3E_POINTER_STATE_RELEASED;
	bool rmbr = (s3ePointerGetState(S3E_POINTER_BUTTON_RIGHTMOUSE) &  S3E_POINTER_STATE_RELEASED) == S3E_POINTER_STATE_RELEASED;

	if (isTitlescreenPhase)
	{
		if (pointerX >= startButtonTarget.x && pointerX < (startButtonTarget.x + startButtonSize.x) && pointerY >= startButtonTarget.y && pointerY < (startButtonTarget.y + startButtonSize.y))
		{
			isStartButtonHovered = true;

			if (lmbr || mmbr || rmbr)
			{
				RestartGame();
			}
		}
		else
		{
			isStartButtonHovered = false;
		}

		if (pointerX >= quitButtonTarget.x && pointerX < (quitButtonTarget.x + quitButtonSize.x) && pointerY >= quitButtonTarget.y && pointerY < (quitButtonTarget.y + quitButtonSize.y))
		{
			isQuitButtonHovered = true;

			if (lmbr || mmbr || rmbr)
			{
				g_QuitGame = true;
			}
		}
		else
		{
			isQuitButtonHovered = false;
		}

		if (s3eKeyboardGetState(s3eKeyEnter) & S3E_KEY_STATE_RELEASED)
		{
			RestartGame();
		}

		if (s3eKeyboardGetState(s3eKeyBackspace) & S3E_KEY_STATE_RELEASED)
		{
			g_QuitGame = true;
		}
	}
	else
	{
		if (!isGetReadyPhase && !isLeveLCompletePhase && !isGameOverPhase)
		{
			HandleInput();
		}

		if (isPausedState)
		{
			if (pointerX >= titlesButtonTarget.x && pointerX < (titlesButtonTarget.x + titlesButtonSize.x) && pointerY >= titlesButtonTarget.y && pointerY < (titlesButtonTarget.y + titlesButtonSize.y))
			{
				isTitlesButtonHovered = true;

				if (lmbr || mmbr || rmbr)
				{
					QuitToTitles();
				}
			}
			else
			{
				isTitlesButtonHovered = false;
			}
			
			if (pointerX >= continueButtonTarget.x && pointerX < (continueButtonTarget.x + continueButtonSize.x) && pointerY >= continueButtonTarget.y && pointerY < (continueButtonTarget.y + continueButtonSize.y))
			{
				isContinueButtonHovered = true;

				if (lmbr || mmbr || rmbr)
				{
					isPausedState = !isPausedState;
				}
			}
			else
			{
				isContinueButtonHovered = false;
			}
		}
		else
		{
			int32 origSurfaceWidth = Iw2DGetSurfaceWidth();
			int32 origSurfaceHeight = Iw2DGetSurfaceHeight();

			int32 halfOrigSurfaceWidth = origSurfaceWidth / 2;
			int32 halfOrigSurfaceHeight = origSurfaceHeight / 2;

			panOffset.x = (float)halfOrigSurfaceWidth - center.x;
			panOffset.y = (float)halfOrigSurfaceHeight - center.y;

			int32 bgCenterTileNr = currentLevel->ReadBgTileNrFromPixels((int32)center.x, (int32)center.y);
			if (bgCenterTileNr <= 0)
			{
				player->Health = 0.0f;
			}

			int32 mgCenterTileNr = currentLevel->ReadMgTileNrFromPixels((int32)center.x, (int32)center.y);
			if (mgCenterTileNr == TILENR_EXIT_SIGN_A && currentLevel->ExitActive && isLeveLCompletePhase == false)
			{
				MoveToNextLevel();
			}

			player->Update();
			currentLevel->Update();

			for (int32 projectileIndex = (int32)playerProjectiles.size() - 1; projectileIndex >= 0; projectileIndex--)
			{
				playerProjectiles[projectileIndex].Update();

				int32 tileNrUnderShot = currentLevel->ReadMgTileNrFromPixels((int32)playerProjectiles[projectileIndex].Position.x, (int32)playerProjectiles[projectileIndex].Position.y);
				if (tileNrUnderShot > 16)
				{
					playerProjectiles[projectileIndex].Lifetime = 0;

					Explosion newExplosion;
					newExplosion.Position = playerProjectiles[projectileIndex].Position;
					newExplosion.ResetLifetime();
					explosions.push_back(newExplosion);
				}

				CIwFVec2 nestHitPosition;
				if (currentLevel->HandleShotHittingNests(playerProjectiles[projectileIndex], nestHitPosition))
				{
					playerProjectiles[projectileIndex].Lifetime = 0;

					Explosion newExplosion;
					newExplosion.Position = playerProjectiles[projectileIndex].Position;
					newExplosion.ResetLifetime();
					explosions.push_back(newExplosion);

					player->Score += 1;
				}

				if (playerProjectiles[projectileIndex].Lifetime <= 0)
				{
					playerProjectiles.erase(projectileIndex);
				}
			}

			for (int32 explosionIndex = (int32)explosions.size() - 1; explosionIndex >= 0; explosionIndex--)
			{
				explosions[explosionIndex].Update();

				if (explosions[explosionIndex].Lifetime <= 0)
				{
					explosions.erase(explosionIndex);
				}
			}

			if (phaseTimeoutCounter > 0 && (isGetReadyPhase || isLeveLCompletePhase || isGameOverPhase))
			{
				phaseTimeoutCounter -= 1;

				if (phaseTimeoutCounter <= 0)
				{
					if (isGameOverPhase)
					{
						QuitToTitles();
					}
					else if (isLeveLCompletePhase)
					{
						player->Score += 5;

						player->Level += 1;
						RestartLevel();						
					}

					isGetReadyPhase = false;
					isGameOverPhase = false;
					isLeveLCompletePhase = false;
				}
			}

			if (!isGameOverPhase && player->Health <= 0.0f)
			{
				isGameOverPhase = true;
				phaseTimeoutCounter = 5 * 60;
			}
		}
	}

	// --- last
	frameCounter++;
	if (frameCounter >= FRAME_RATE_HZ)
	{
		frameCounter = 0;
	}
}

void Application::Render()
{
	Iw2DSurfaceClear(clearColour);

	Iw2DSetColour(whiteColour);

	if (isTitlescreenPhase)
	{
		int32 origSurfaceWidth = Iw2DGetSurfaceWidth();
		int32 origSurfaceHeight = Iw2DGetSurfaceHeight();

		int32 halfOrigSurfaceWidth = origSurfaceWidth / 2;
		int32 halfOrigSurfaceHeight = origSurfaceHeight / 2;

		CIwFVec2 target(0.0f, 0.0f);
		CIwFVec2 targetSize((float)origSurfaceWidth, (float)origSurfaceHeight);
		CIwFVec2 source(0.0f, 0.0f);
		CIwFVec2 sourceRegion(512, 288);
		Iw2DDrawImageRegion(titleImage, target, targetSize, source, sourceRegion);

		// start button
		if (!isStartButtonHovered)
		{
			Iw2DDrawImageRegion(tileImage, startButtonTarget, startButtonSourceNormal, startButtonSize);
		}
		else
		{
			Iw2DDrawImageRegion(tileImage, startButtonTarget, startButtonSourceHovered, startButtonSize);
		}

		// quit button
		if (!isQuitButtonHovered)
		{
			Iw2DDrawImageRegion(tileImage, quitButtonTarget, quitButtonSourceNormal, quitButtonSize);
		}
		else
		{
			Iw2DDrawImageRegion(tileImage, quitButtonTarget, quitButtonSourceHovered, quitButtonSize);
		}
	}
	else
	{
		currentLevel->Render();
		player->Render();

		for (uint32 projectileIndex = 0; projectileIndex < playerProjectiles.size(); projectileIndex++)
		{
			playerProjectiles[projectileIndex].Render();
		}

		for (uint32 explosionIndex = 0; explosionIndex < explosions.size(); explosionIndex++)
		{
			explosions[explosionIndex].Render();
		}

		RenderHealthBar();

		// ---
		char nestsUiTxt[MAX_TEXT_LEN];
		sprintf(nestsUiTxt, "Nests: %02d", std::min((uint32)99, currentLevel->CountNests()));
		std::string nestsRenderText = nestsUiTxt;

		CIwFVec2 topcenter((float)Iw2DGetSurfaceWidth() / 2 - 80.0f, 8.0f);
		CIwFVec2 region(160.0f, 16.0f);
		RenderText16(nestsRenderText, topcenter, region, whiteColour, blackColour, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP);

		// ---
		char scoreUiTxt[MAX_TEXT_LEN];
		sprintf(scoreUiTxt, "SCORE %07d", std::min((uint32)9999999, (uint32)player->Score));
		std::string scoreRenderText = scoreUiTxt;

		CIwFVec2 topRight((float)Iw2DGetSurfaceWidth() - 160.0f - 8.0f, 8.0f);
		CIwFVec2 regionScore(160.0f, 16.0f);
		RenderText12(scoreRenderText, topRight, regionScore, whiteColour, blackColour, IW_2D_FONT_ALIGN_RIGHT, IW_2D_FONT_ALIGN_TOP);

		// ---
		char levelUiTxt[MAX_TEXT_LEN];
		sprintf(levelUiTxt, "LEVEL %02d", std::min((uint32)999, (uint32)player->Level));
		std::string levelRenderText = levelUiTxt;

		CIwFVec2 centerBottom((float)Iw2DGetSurfaceWidth() / 2 - 80.0f, (float)Iw2DGetSurfaceHeight() - 24.0f);
		CIwFVec2 regionLevel(160.0f, 16.0f);
		RenderText12(levelRenderText, centerBottom, regionLevel, greenColour, blackColour, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_TOP);

		if (isGetReadyPhase || isGameOverPhase || isLeveLCompletePhase || isPausedState)
		{
			uint blinkTextIndex = frameCounter / QUARTER_FRAME_RATE_HZ;
			if (blinkTextIndex == 0 || blinkTextIndex == 2)
			{
				char getReadyUiTxt[MAX_TEXT_LEN];
				if (isGetReadyPhase)
				{
					sprintf(getReadyUiTxt, "GET READY!");
				}
				else if (isGameOverPhase)
				{
					sprintf(getReadyUiTxt, "GAME OVER!");
				}
				else if (isLeveLCompletePhase)
				{
					sprintf(getReadyUiTxt, "LEVEL COMPLETE!");
				}
				else 
				{
					sprintf(getReadyUiTxt, "PAUSED!");
				}
				std::string getReadyRenderText = getReadyUiTxt;

				CIwFVec2 center1((float)(Iw2DGetSurfaceWidth() / 2) - 80.0f, (float)(Iw2DGetSurfaceHeight() / 2) - 18.0f);
				CIwFVec2 regionCenter1(160.0f, 36.0f);
				RenderText16(getReadyUiTxt, center1, regionCenter1, whiteColour, blackColour, IW_2D_FONT_ALIGN_CENTRE, IW_2D_FONT_ALIGN_CENTRE);
			}

			if (isPausedState)
			{
				Iw2DSetColour(whiteColour);

				// titles button
				if (!isTitlesButtonHovered)
				{
					Iw2DDrawImageRegion(tileImage, titlesButtonTarget, titlesButtonSourceNormal, titlesButtonSize);
				}
				else
				{
					Iw2DDrawImageRegion(tileImage, titlesButtonTarget, titlesButtonSourceHovered, titlesButtonSize);
				}

				// continue button
				if (!isContinueButtonHovered)
				{
					Iw2DDrawImageRegion(tileImage, continueButtonTarget, continueButtonSourceNormal, continueButtonSize);
				}
				else
				{
					Iw2DDrawImageRegion(tileImage, continueButtonTarget, continueButtonSourceHovered, continueButtonSize);
				}
			}
		}
	}
}

void Application::Shutdown()
{
}

void Application::RestartGame()
{
	isTitlescreenPhase = false;

	if (currentLevel != NULL)
	{
		delete currentLevel;
		currentLevel = NULL;
	}
	currentLevel = new Level();

	frameCounter = 0;

	if (player != NULL)
	{
		delete player;
		player = NULL;
	}
	player = new Player();

	playerRampSpeed = 0.0f;

	RestartLevel();
}

void Application::RestartLevel()
{
	isGetReadyPhase = true;
	isLeveLCompletePhase = false;
	isGameOverPhase = false;

	phaseTimeoutCounter = 180;

	isPausedState = false;

	if ((player->Level - 1) % 2 == 0)
	{
		currentLevel->LoadFromFile("testlevel1_bg.csv", "testlevel1_mg.csv", 24, 16);
	}
	else
	{
		currentLevel->LoadFromFile("testlevel2_bg.csv", "testlevel2_mg.csv", 24, 48);
	}

	CIwVec2 playerSpawnCoords = currentLevel->ReadPlayerSpawnCoordinates();
	center.x = (float)(playerSpawnCoords.x * TILE_WIDTH) + HALF_TILE_WIDTH;
	center.y = (float)(playerSpawnCoords.y * TILE_HEIGHT) + HALF_TILE_HEIGHT;

	currentLevel->ExitActive = false;

	lastShotCount = PROJECTILE_SHOT_INTERVAL;
}

void Application::QuitToTitles()
{
	isTitlescreenPhase = true;
}

CIw2DImage* Application::GetTileImage()
{
	return tileImage;
}

uint32 Application::GetTilesPerImageWidth()
{
	return tilesPerImageWidth;
}

CIwFVec2 Application::GetPanOffset()
{
	return panOffset;
}

void Application::HandleInput()
{
	bool playerFaceUp = false;
	bool playerFaceDown = false;
	bool playerFaceLeft = false;
	bool playerFaceRight = false;

	bool playerAccelerated = false;

	// 1
	CIwFVec2 playerPrevMoveVector(0.0f, 0.0f);
	if (playerRampSpeed > 0.0f)
	{
		switch (player->Heading)
		{
		case PLAYER_HEADING_SOUTH:
			playerPrevMoveVector.y = playerRampSpeed;
			break;
		case PLAYER_HEADING_SOUTH_WEST:
			playerPrevMoveVector.x = -playerRampSpeed;
			playerPrevMoveVector.y = playerRampSpeed;
			break;
		case PLAYER_HEADING_WEST:
			playerPrevMoveVector.x = -playerRampSpeed;
			break;
		case PLAYER_HEADING_NORTH_WEST:
			playerPrevMoveVector.x = -playerRampSpeed;
			playerPrevMoveVector.y = -playerRampSpeed;
			break;
		case PLAYER_HEADING_NORTH:
			playerPrevMoveVector.y = -playerRampSpeed;
			break;
		case PLAYER_HEADING_NORTH_EAST:
			playerPrevMoveVector.x = playerRampSpeed;
			playerPrevMoveVector.y = -playerRampSpeed;
			break;
		case PLAYER_HEADING_EAST:
			playerPrevMoveVector.x = playerRampSpeed;
			break;
		case PLAYER_HEADING_SOUTH_EAST:
			playerPrevMoveVector.x = playerRampSpeed;
			playerPrevMoveVector.y = playerRampSpeed;
			break;
		default:
			break;
		}
	}

	// 2a
	CIwFVec2 playerMoveVector(0.0f, 0.0f);
	if (s3eKeyboardGetState(s3eKeyW) & S3E_KEY_STATE_DOWN)
	{
		playerMoveVector.y -= PLAYER_MAX_MOVEMENT_SPEED;
		playerFaceUp = true;

		playerAccelerated = true;
	}
	else if (s3eKeyboardGetState(s3eKeyS) & S3E_KEY_STATE_DOWN)
	{
		playerMoveVector.y += PLAYER_MAX_MOVEMENT_SPEED;
		playerFaceDown = true;

		playerAccelerated = true;
	}

	if (s3eKeyboardGetState(s3eKeyA) & S3E_KEY_STATE_DOWN)
	{
		playerMoveVector.x -= PLAYER_MAX_MOVEMENT_SPEED;
		playerFaceLeft = true;

		playerAccelerated = true;
	}
	else if (s3eKeyboardGetState(s3eKeyD) & S3E_KEY_STATE_DOWN)
	{
		playerMoveVector.x += PLAYER_MAX_MOVEMENT_SPEED;
		playerFaceRight = true;

		playerAccelerated = true;
	}

	if (playerAccelerated)
	{
		playerRampSpeed += PLAYER_ACCELERATION;
		if (playerRampSpeed >= PLAYER_MAX_MOVEMENT_SPEED)
		{
			playerRampSpeed = PLAYER_MAX_MOVEMENT_SPEED;
		}
	}
	else
	{
		playerRampSpeed -= PLAYER_DECELERATION;
		if (playerRampSpeed < 0.0f)
		{
			playerRampSpeed = 0.0f;
		}
	}

	// 2b
	CIwFVec2 finalMoveVector = playerPrevMoveVector + playerMoveVector;
	if (!finalMoveVector.IsZero())
	{
		finalMoveVector.Normalise();
	}
	finalMoveVector *= playerRampSpeed;

	CIwFVec2 proposedNewPosition = center + finalMoveVector;

	bool playerMovementUpBlocked = false;
	bool playerMovementRightBlocked = false;
	bool playerMovementDownBlocked = false;
	bool playerMovementLeftBlocked = false;

	int32 topLeftTileNr = currentLevel->ReadMgTileNrFromPixels((int32)(proposedNewPosition.x - PLAYER_COLLISION_RADIUS), (int32)(proposedNewPosition.y - PLAYER_COLLISION_RADIUS));
	int32 topRightTileNr = currentLevel->ReadMgTileNrFromPixels((int32)(proposedNewPosition.x + PLAYER_COLLISION_RADIUS), (int32)(proposedNewPosition.y - PLAYER_COLLISION_RADIUS));
	if ((topLeftTileNr >= MIN_BLOCKED_TILENR && topLeftTileNr <= MAX_BLOCKED_TILENR) && (topRightTileNr >= MIN_BLOCKED_TILENR && topRightTileNr <= MAX_BLOCKED_TILENR))
	{
		playerMovementUpBlocked = true;
	}

	int32 bottomLeftTileNr = currentLevel->ReadMgTileNrFromPixels((int32)(proposedNewPosition.x - PLAYER_COLLISION_RADIUS), (int32)(proposedNewPosition.y + PLAYER_COLLISION_RADIUS));
	int32 bottomRightTileNr = currentLevel->ReadMgTileNrFromPixels((int32)(proposedNewPosition.x + PLAYER_COLLISION_RADIUS), (int32)(proposedNewPosition.y + PLAYER_COLLISION_RADIUS));
	if ((bottomLeftTileNr >= MIN_BLOCKED_TILENR && bottomLeftTileNr <= MAX_BLOCKED_TILENR) && (bottomRightTileNr >= MIN_BLOCKED_TILENR && bottomRightTileNr <= MAX_BLOCKED_TILENR))
	{
		playerMovementDownBlocked = true;
	}

	int32 leftTopTileNr = currentLevel->ReadMgTileNrFromPixels((int32)(proposedNewPosition.x - PLAYER_COLLISION_RADIUS), (int32)(proposedNewPosition.y - PLAYER_COLLISION_RADIUS));
	int32 leftBottomTileNr = currentLevel->ReadMgTileNrFromPixels((int32)(proposedNewPosition.x - PLAYER_COLLISION_RADIUS), (int32)(proposedNewPosition.y + PLAYER_COLLISION_RADIUS));
	if ((leftTopTileNr >= MIN_BLOCKED_TILENR && topRightTileNr <= MAX_BLOCKED_TILENR) && (leftBottomTileNr >= MIN_BLOCKED_TILENR && leftBottomTileNr <= MAX_BLOCKED_TILENR))
	{
		playerMovementLeftBlocked = true;
	}

	int32 rightTopTileNr = currentLevel->ReadMgTileNrFromPixels((int32)(proposedNewPosition.x + PLAYER_COLLISION_RADIUS), (int32)(proposedNewPosition.y - PLAYER_COLLISION_RADIUS));
	int32 rightBottomTileNr = currentLevel->ReadMgTileNrFromPixels((int32)(proposedNewPosition.x + PLAYER_COLLISION_RADIUS), (int32)(proposedNewPosition.y + PLAYER_COLLISION_RADIUS));
	if ((rightTopTileNr >= MIN_BLOCKED_TILENR && rightTopTileNr <= MAX_BLOCKED_TILENR) && (rightBottomTileNr >= MIN_BLOCKED_TILENR && rightBottomTileNr <= MAX_BLOCKED_TILENR))
	{
		playerMovementRightBlocked = true;
	}

	if (playerMovementUpBlocked && finalMoveVector.y < 0.0f)
	{
		finalMoveVector.y = 0.0f;
	}
	else if (playerMovementDownBlocked && finalMoveVector.y > 0.0f)
	{
		finalMoveVector.y = 0.0f;
	}

	if (playerMovementLeftBlocked && finalMoveVector.x < 0.0f)
	{
		finalMoveVector.x = 0.0f;
	}
	else if (playerMovementRightBlocked && finalMoveVector.x > 0.0f)
	{
		finalMoveVector.x = 0.0f;
	}

	if (!finalMoveVector.IsZero())
	{
		finalMoveVector.Normalise();
	}
	finalMoveVector *= playerRampSpeed;

	center += finalMoveVector;

	// 3
	if (playerFaceUp)
	{
		if (playerFaceLeft)
		{
			player->Heading = PLAYER_HEADING_NORTH_WEST;
		}
		else if (playerFaceRight)
		{
			player->Heading = PLAYER_HEADING_NORTH_EAST;
		}
		else
		{
			player->Heading = PLAYER_HEADING_NORTH;
		}
	}
	else if (playerFaceDown)
	{
		if (playerFaceLeft)
		{
			player->Heading = PLAYER_HEADING_SOUTH_WEST;

		}
		else if (playerFaceRight)
		{
			player->Heading = PLAYER_HEADING_SOUTH_EAST;

		}
		else
		{
			player->Heading = PLAYER_HEADING_SOUTH;
		}
	}
	else
	{
		if (playerFaceLeft)
		{
			player->Heading = PLAYER_HEADING_WEST;
		}
		else if (playerFaceRight)
		{
			player->Heading = PLAYER_HEADING_EAST;
		}
	}

	// 3a
	int32  pointerX = s3ePointerGetX();
	int32  pointerY = s3ePointerGetY();

	bool lmb = (s3ePointerGetState(S3E_POINTER_BUTTON_LEFTMOUSE) &  S3E_POINTER_STATE_DOWN) == S3E_POINTER_STATE_DOWN;
	if (lmb && lastShotCount >= PROJECTILE_SHOT_INTERVAL)
	{
		int32 origSurfaceWidth = Iw2DGetSurfaceWidth();
		int32 origSurfaceHeight = Iw2DGetSurfaceHeight();

		int32 halfOrigSurfaceWidth = origSurfaceWidth / 2;
		int32 halfOrigSurfaceHeight = origSurfaceHeight / 2;

		Projectile newProjectile;
		newProjectile.Position = center;
		newProjectile.Direction = CIwFVec2((float)(pointerX - halfOrigSurfaceWidth), (float)(pointerY - halfOrigSurfaceHeight));
		if (!newProjectile.Direction.IsZero())
		{
			newProjectile.Direction.Normalise();
		}
		newProjectile.Speed = PROJECTILE_SPEED;
		newProjectile.Lifetime = PROJECTILE_LIFETIME;
		playerProjectiles.push_back(newProjectile);

		lastShotCount = 0;
	}
	else
	{
		lastShotCount++;
	}

	// 4
	if (s3eKeyboardGetState(s3eKeyComma) & S3E_KEY_STATE_RELEASED)
	{
		player->Level -= 1;
		RestartLevel();
	}
	else if (s3eKeyboardGetState(s3eKeyPeriod) & S3E_KEY_STATE_RELEASED)
	{
		player->Level += 1;
		RestartLevel();
	}

	if (s3eKeyboardGetState(s3eKeyEsc) & S3E_KEY_STATE_RELEASED)
	{
		isPausedState = !isPausedState;
	}

	if (s3eKeyboardGetState(s3eKeyBackspace) & S3E_KEY_STATE_RELEASED)
	{
		if (isPausedState)
		{
			QuitToTitles();
		}
	}

	if (s3eKeyboardGetState(s3eKeyP) & S3E_KEY_STATE_RELEASED)
	{
		if (!isPausedState)
		{
			player->Health = 0;
		}
	}
}

void Application::RenderText12(std::string text, CIwFVec2 position, CIwFVec2 region, CIwColour frontColour, CIwColour backColour, CIw2DFontAlign horzAlign, CIw2DFontAlign vertAlign)
{
	Iw2DSetFont(atariFont12);
	RenderText(text, position, region, frontColour, backColour, horzAlign, vertAlign);
}

void Application::RenderText16(std::string text, CIwFVec2 position, CIwFVec2 region, CIwColour frontColour, CIwColour backColour, CIw2DFontAlign horzAlign, CIw2DFontAlign vertAlign)
{
	Iw2DSetFont(atariFont16);
	RenderText(text, position, region, frontColour, backColour, horzAlign, vertAlign);
}

void Application::RenderText36(std::string text, CIwFVec2 position, CIwFVec2 region, CIwColour frontColour, CIwColour backColour, CIw2DFontAlign horzAlign, CIw2DFontAlign vertAlign)
{
	Iw2DSetFont(atariFont36);
	RenderText(text, position, region, frontColour, backColour, horzAlign, vertAlign);
}

void Application::RenderText(std::string text, CIwFVec2 position, CIwFVec2 region, CIwColour frontColour, CIwColour backColour, CIw2DFontAlign horzAlign, CIw2DFontAlign vertAlign)
{
	if (backColour != NULL)
	{
		CIwFVec2 lower(position.x + TEXT_SHADOW_OFFSET_X, position.y + TEXT_SHADOW_OFFSET_Y);
		Iw2DSetColour(backColour);
		Iw2DDrawString(text.c_str(), lower, region, horzAlign, vertAlign);
	}

	Iw2DSetColour(frontColour);
	Iw2DDrawString(text.c_str(), position, region, horzAlign, vertAlign);
}

void Application::RenderHealthBar()
{
	float targetX = 8.0f;
	float targetY = 8.0f;
	CIwFVec2 target(targetX, targetY);
	CIwFVec2 dimension(64, 16);
	CIwFVec2 source(0.0f, 496.0f);
	Iw2DDrawImageRegion(tileImage, target, source, dimension);

	float target1X = targetX + 20.0f;
	float target1Y = targetY + 4.0f;
	CIwFVec2 target1(target1X, target1Y);
	CIwFVec2 dimension1(41.0f * (player->Health / 100.0f), 8.0f);
	CIwFVec2 source1(64.0f, 496.0f);
	Iw2DDrawImageRegion(tileImage, target1, source1, dimension1);
}


uint32 Application::GetFrameCounter()
{
	return frameCounter;
}

uint32 Application::GetPlayerLevel()
{
	return player->Level;
}

Level* Application::GetCurrentLevel()
{
	return currentLevel;
}

void Application::MoveToNextLevel()
{
	isLeveLCompletePhase = true;
	phaseTimeoutCounter = 5 * 60;
}
