#ifndef _APPLICATION_H_
#define _APPLICATION_H_

#include "Constants.h"

#include "Level.h"
#include "Player.h"
#include "Projectile.h"
#include "Explosion.h"

class Application
{
public:
	Application();
	~Application();

	void Init();
	void Update();
	void Render();
	void Shutdown();

	CIw2DImage* GetTileImage();
	uint32 GetTilesPerImageWidth();
	CIwFVec2 GetPanOffset();

	void RenderText12(std::string text, CIwFVec2 position, CIwFVec2 region, CIwColour frontColour, CIwColour backColour, CIw2DFontAlign horzAlign, CIw2DFontAlign vertAlign);
	void RenderText16(std::string text, CIwFVec2 position, CIwFVec2 region, CIwColour frontColour, CIwColour backColour, CIw2DFontAlign horzAlign, CIw2DFontAlign vertAlign);
	void RenderText36(std::string text, CIwFVec2 position, CIwFVec2 region, CIwColour frontColour, CIwColour backColour, CIw2DFontAlign horzAlign, CIw2DFontAlign vertAlign);

	uint32 GetFrameCounter();
	uint32 GetPlayerLevel();

	Level* GetCurrentLevel();

	void MoveToNextLevel();

private:
	void HandleInput();

	void RenderText(std::string text, CIwFVec2 position, CIwFVec2 region, CIwColour frontColour, CIwColour backColour, CIw2DFontAlign horzAlign, CIw2DFontAlign vertAlign);
	void RenderHealthBar();

	void RestartGame();
	void RestartLevel();
	void QuitToTitles();

private:
	CIwFVec2 panOffset;

	CIw2DImage* titleImage;

	CIw2DImage* tileImage;
	uint32 tilesPerImageWidth;

	Level* currentLevel;
	CIwFVec2 center;

	CIw2DFont* atariFont12;
	CIw2DFont* atariFont16;
	CIw2DFont* atariFont36;

	uint32 frameCounter;
	
	Player* player;
	float playerRampSpeed;

	bool isTitlescreenPhase;
	bool isGetReadyPhase;
	bool isLeveLCompletePhase;
	bool isGameOverPhase;
	bool isPausedState;
	uint32 phaseTimeoutCounter;

	CIwFVec2 startButtonSourceNormal;
	CIwFVec2 startButtonSourceHovered;
	CIwFVec2 startButtonTarget;
	CIwFVec2 startButtonSize;
	bool isStartButtonHovered;

	CIwFVec2 quitButtonSourceNormal;
	CIwFVec2 quitButtonSourceHovered;
	CIwFVec2 quitButtonTarget;
	CIwFVec2 quitButtonSize;
	bool isQuitButtonHovered;

	CIwFVec2 titlesButtonSourceNormal;
	CIwFVec2 titlesButtonSourceHovered;
	CIwFVec2 titlesButtonTarget;
	CIwFVec2 titlesButtonSize;
	bool isTitlesButtonHovered;

	CIwFVec2 continueButtonSourceNormal;
	CIwFVec2 continueButtonSourceHovered;
	CIwFVec2 continueButtonTarget;
	CIwFVec2 continueButtonSize;
	bool isContinueButtonHovered;

	CIwArray<Projectile> playerProjectiles;
	uint32 lastShotCount;

	CIwArray<Explosion> explosions;
};

#endif