#include "Level.h"
#include "Application.h"

extern Application* g_App;

Level::Level()
{
	validLevelLoaded = false;

	blinkIndex = 0; 
	wiggleIndex = 0;

	ExitActive = false;
}

Level::~Level()
{
	tilesBg.clear();
	tilesMg.clear();
}

void Level::LoadFromFile(std::string filenameBg, std::string filenameMg, uint32 newWidth, uint32 newHeight)
{
	validLevelLoaded = false;

	bool validMap1Loaded = false;
	bool validMap2Loaded = false;

	nosTilesWidth = newWidth;
	nosTilesHeight = newHeight;

	std::string openOptions = "r";

	// ---
	tilesBg.clear();

	s3eFile* mapFileBg = s3eFileOpen(filenameBg.c_str(), openOptions.c_str());
	if (mapFileBg != NULL)
	{
		int32 lineIndex = 0;
		char inputLine[MAP_LINE_INPUT_MAXLEN];

		while (s3eFileReadString(inputLine, MAP_LINE_INPUT_MAXLEN, mapFileBg) != NULL)
		{
			if (lineIndex == nosTilesHeight)
			{
				break;
			}

			CIwArray<std::string> splits = split(std::string(inputLine), ',');
			for (uint32 columnIndex = 0; columnIndex < nosTilesWidth; columnIndex++)
			{
				std::string currValueString = splits[columnIndex];
				int32 currValue = (int32)atoi(currValueString.c_str());
				tilesBg.push_back(currValue);
			}

			lineIndex++;
		}
		s3eFileClose(mapFileBg);

		validMap1Loaded = true;
	}

	// ---
	tilesMg.clear();
	allSpawnNestsTileCoordinates.clear();

	s3eFile* mapFileMg = s3eFileOpen(filenameMg.c_str(), openOptions.c_str());
	if (mapFileMg != NULL)
	{
		int32 lineIndex = 0;
		char inputLine[MAP_LINE_INPUT_MAXLEN];

		while (s3eFileReadString(inputLine, MAP_LINE_INPUT_MAXLEN, mapFileMg) != NULL)
		{
			if (lineIndex == nosTilesHeight)
			{
				break;
			}

			CIwArray<std::string> splits = split(std::string(inputLine), ',');
			for (uint32 columnIndex = 0; columnIndex < nosTilesWidth; columnIndex++)
			{
				std::string currValueString = splits[columnIndex];
				int32 currValue = (int32)atoi(currValueString.c_str());

				// --
				switch (currValue)
				{
				case TILENR_PLAYER_SPAWN:
					playerSpawnTileCoords.x = columnIndex;
					playerSpawnTileCoords.y = lineIndex;
					currValue = EMPTY_TILE_NR;
					break;
				case TILENR_NEST_SPAWN:
					allSpawnNestsTileCoordinates.push_back(CIwVec2(columnIndex, lineIndex));
					currValue = EMPTY_TILE_NR;
					break;
				default:
					break;
				}

				tilesMg.push_back(currValue);
			}

			lineIndex++;
		}

		validMap2Loaded = true;

		s3eFileClose(mapFileMg);
	}

	SpawnStartingNests();

	validLevelLoaded = validMap1Loaded & validMap2Loaded;
}

CIwArray<std::string>& Level::split(const std::string &s, char delim, CIwArray<std::string> &elems)
{
	std::stringstream ss(s);
	std::string item;

	while (std::getline(ss, item, delim))
	{
		elems.push_back(item);
	}

	return elems;
}


CIwArray<std::string> Level::split(const std::string &s, char delim)
{
	CIwArray<std::string> elems;
	split(s, delim, elems);
	return elems;
}

void Level::Update()
{
	blinkIndex = g_App->GetFrameCounter() / (FRAME_RATE_HZ / 4);

	if (g_App->GetFrameCounter() == FRAME_RATE_HZ - 1)
	{
		wiggleIndex = wiggleIndex == 0 ? 1 : 0;
	}

	if (!ExitActive && validLevelLoaded && spawnedNestsTileCoordinates.size() == 0)
	{
		ExitActive = true;
	}
}

void Level::Render()
{
	RenderTiles();
	RenderSpawnedNests();
}

void Level::RenderTiles()
{
	int32 origSurfaceWidth = Iw2DGetSurfaceWidth();
	int32 origSurfaceHeight = Iw2DGetSurfaceHeight();

	for (uint32 y = 0; y < nosTilesHeight; y++)
	{
		for (uint32 x = 0; x < nosTilesWidth; x++)
		{
			// 1
			int32 tileNr = ReadBgTileNr(x, y);
			if (tileNr > 0)
			{
				float targetX = ((float)x * TILE_WIDTH);
				targetX += g_App->GetPanOffset().x;
				float targetY = ((float)y * TILE_HEIGHT);
				targetY += g_App->GetPanOffset().y;
				CIwFVec2 target(targetX, targetY);

				CIwFVec2 dimension((float)TILE_WIDTH, (float)TILE_HEIGHT);

				if (target.x > -dimension.x && target.x < origSurfaceWidth && target.y > -dimension.y && target.y < origSurfaceHeight)
				{
					float sourceX = (float)(tileNr % g_App->GetTilesPerImageWidth()) * TILE_WIDTH;
					float sourceY = (float)(tileNr / g_App->GetTilesPerImageWidth()) * TILE_HEIGHT;
					CIwFVec2 source(sourceX, sourceY);

					Iw2DDrawImageRegion(g_App->GetTileImage(), target, source, dimension);
				}
			}

			// 2
			tileNr = ReadMgTileNr(x, y);
			if (tileNr > 0)
			{
				// --
				int32 origTileNr = tileNr;
				switch (tileNr)
				{
				case TILENR_EXIT_SIGN_A:
					tileNr += blinkIndex % 2;
					break;
				default:
					break;
				}

				if (origTileNr != TILENR_EXIT_SIGN_A || (origTileNr == TILENR_EXIT_SIGN_A && ExitActive))
				{
					float targetX = ((float)x * TILE_WIDTH);
					targetX += g_App->GetPanOffset().x;
					float targetY = ((float)y * TILE_HEIGHT);
					targetY += g_App->GetPanOffset().y;
					CIwFVec2 target(targetX, targetY);

					CIwFVec2 dimension((float)TILE_WIDTH, (float)TILE_HEIGHT);

					if (target.x > -dimension.x && target.x < origSurfaceWidth && target.y > -dimension.y && target.y < origSurfaceHeight)
					{
						float sourceX = (float)(tileNr % g_App->GetTilesPerImageWidth()) * TILE_WIDTH;
						float sourceY = (float)(tileNr / g_App->GetTilesPerImageWidth()) * TILE_HEIGHT;
						CIwFVec2 source(sourceX, sourceY);

						Iw2DDrawImageRegion(g_App->GetTileImage(), target, source, dimension);
					}
				}
			}
		}
	}
}

void Level::RenderSpawnedNests()
{
	int32 origSurfaceWidth = Iw2DGetSurfaceWidth();
	int32 origSurfaceHeight = Iw2DGetSurfaceHeight();

	int32 tileNr = wiggleIndex == 0 ? TILENR_NEST_FULL_A : TILENR_NEST_FULL_B;

	for (uint index = 0; index < spawnedNestsTileCoordinates.size(); index++)
	{
		CIwVec2 currentTilePosition = spawnedNestsTileCoordinates[index];

		float targetX = ((float)currentTilePosition.x * TILE_WIDTH);
		targetX += g_App->GetPanOffset().x;
		float targetY = ((float)currentTilePosition.y * TILE_HEIGHT);
		targetY += g_App->GetPanOffset().y;
		CIwFVec2 target(targetX, targetY);

		CIwFVec2 dimension((float)TILE_WIDTH, (float)TILE_HEIGHT);

		if (target.x > -dimension.x && target.x < origSurfaceWidth && target.y > -dimension.y && target.y < origSurfaceHeight)
		{
			float sourceX = (float)(tileNr % g_App->GetTilesPerImageWidth()) * TILE_WIDTH;
			float sourceY = (float)(tileNr / g_App->GetTilesPerImageWidth()) * TILE_HEIGHT;
			CIwFVec2 source(sourceX, sourceY);

			Iw2DDrawImageRegion(g_App->GetTileImage(), target, source, dimension);
		}
	}
}

int32 Level::ReadBgTileNr(int32 tileX, int32 tileY)
{
	if (tileX < 0 || tileX >= (int32)nosTilesWidth || tileY < 0 || tileY >= (int32)nosTilesHeight)
	{
		return 0;
	}

	return tilesBg[(tileY * nosTilesWidth) + tileX];
}

int32 Level::ReadBgTileNrFromPixels(int32 pixelX, int32 pixelY)
{
	return ReadBgTileNr(pixelX / TILE_WIDTH, pixelY / TILE_HEIGHT);
}

int32 Level::ReadMgTileNr(int32 tileX, int32 tileY)
{
	if (tileX < 0 || tileX >= (int32)nosTilesWidth || tileY < 0 || tileY >= (int32)nosTilesHeight)
	{
		return 0;
	}

	return tilesMg[(tileY * nosTilesWidth) + tileX];
}

int32 Level::ReadMgTileNrFromPixels(int32 pixelX, int32 pixelY)
{
	return ReadMgTileNr(pixelX / TILE_WIDTH, pixelY / TILE_HEIGHT);
}

CIwVec2 Level::ReadPlayerSpawnCoordinates()
{
	return playerSpawnTileCoords;
}

void Level::SpawnStartingNests()
{
	spawnedNestsTileCoordinates.clear();

	CIwArray<uint32> spawnIndexes;
	for (uint32 index = 0; index < allSpawnNestsTileCoordinates.size(); index++)
	{
		spawnIndexes.push_back(index);
	}
	
	uint32 nosNestsToSpawn = (uint32)std::min(BASIC_NEST_SPAWN_PER_LEVEL + (g_App->GetPlayerLevel() - 1), allSpawnNestsTileCoordinates.size());
	CIwArray<uint32> chosenIndexes;
	for (uint32 index = 0; index < nosNestsToSpawn; index++)
	{
		uint32 chosenIndex = -1;
		do
		{
			chosenIndex = IwRandRange(spawnIndexes.size());
		} while (std::find(chosenIndexes.begin(), chosenIndexes.end(), chosenIndex) != chosenIndexes.end());

		chosenIndexes.push_back(chosenIndex);
		spawnedNestsTileCoordinates.push_back(allSpawnNestsTileCoordinates[chosenIndex]);
	}
}

bool Level::HandleShotHittingNests(Projectile projectile, CIwFVec2& nestPosition)
{
	uint32 projectileTileX = (uint32)(projectile.Position.x / TILE_WIDTH);
	uint32 projectileTileY = (uint32)(projectile.Position.y / TILE_HEIGHT);

	for (int32 index = spawnedNestsTileCoordinates.size() - 1; index >= 0; index--)
	{
		if (projectileTileX == spawnedNestsTileCoordinates[index].x && projectileTileY == spawnedNestsTileCoordinates[index].y)
		{
			nestPosition.x = (float)(spawnedNestsTileCoordinates[index].x * TILE_WIDTH + HALF_TILE_WIDTH);
			nestPosition.y = (float)(spawnedNestsTileCoordinates[index].y * TILE_HEIGHT + HALF_TILE_HEIGHT);

			spawnedNestsTileCoordinates.erase(index);
			return true;
		}
	}

	return false;
}

uint32 Level::CountNests()
{
	return spawnedNestsTileCoordinates.size();
}
